/*
 * Noise generation for PuTTY's cryptographic random number
 * generator.
 */

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/resource.h>

#include "putty.h"
#include "storage.h"

#include "vmsrmi.h"

/*
 * This function is called once, at PuTTY startup. It will do some
 * slightly silly things such as fetching an entire process listing
 * and scanning /tmp, load the saved random seed from disk, and
 * also read 32 bytes out of /dev/urandom.
 */

void noise_get_heavy(void (*func) (void *, int))
{
    unsigned char *buf;
    FILE *fp;
    int ret;

    buf = calloc( 1, 512 );

    get_randombytes( &buf , 512 );
    func( buf, 512 );

    read_random_seed(func);
    random_save_seed();
}

void random_save_seed(void)
{
    int len;
    void *data;

    if (random_active) {
	random_get_savedata(&data, &len);
	write_random_seed(data, len);
	sfree(data);
    }
}

/*
 * This function is called every time the random pool needs
 * stirring, and will acquire the system time.
 */
void noise_get_light(void (*func) (void *, int))
{
    struct timeval tv;
    gettimeofday(&tv, NULL);
    func(&tv, sizeof(tv));
}

/*
 * This function is called on a timer, and grabs as much changeable
 * system data as it can quickly get its hands on.
 */
void noise_regular(void)
{
    int fd;
    int ret;
    unsigned char *buf;
    struct rusage rusage;

    buf = calloc( 1, 512 );

    get_randombytes( &buf , 512 );
    random_add_noise( (char *)buf, 512 );

}

/*
 * This function is called on every keypress or mouse move, and
 * will add the current time to the noise pool. It gets the scan
 * code or mouse position passed in, and adds that too.
 */
void noise_ultralight(unsigned long data)
{
    struct timeval tv;
    gettimeofday(&tv, NULL);
    random_add_noise(&tv, sizeof(tv));
    random_add_noise(&data, sizeof(data));
}
